/*
common.c

Copyright (C) 1999 Lars Brinkhoff.  See COPYING for terms and conditions.

Code common to both htc and hts.
*/

#include <stdio.h>
#include <fcntl.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/poll.h>

#include "tunnel.h"
#include "common.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

int
server_socket (int port)
{
  struct sockaddr_in address;
  int s;

  s = socket (PF_INET, SOCK_STREAM, 0);
  if (s == -1)
    return -1;
  
  address.sin_family = PF_INET;
  address.sin_port = htons (port);
  address.sin_addr.s_addr = INADDR_ANY;
  
  if (bind (s, (struct sockaddr *)&address, sizeof (address)) == -1)
    {
      close (s);
      return -1;
    }

  if (listen (s, 1) == -1)
    {
      close (s);
      return -1;
    } 

  return s;
}

int
set_address (struct sockaddr_in *address, const char *host, int port)
{
  address->sin_family = PF_INET;
  address->sin_port = htons (port);
  address->sin_addr.s_addr = inet_addr (host);

  if (address->sin_addr.s_addr == INADDR_NONE)
    {
      struct hostent *hostent;

      hostent = gethostbyname (host);
      if (hostent == 0)
	  return -1;

      memcpy(&address->sin_addr.s_addr,
	     hostent->h_addr,
	     hostent->h_length);
    }

  return 0;
}

int
open_device (char *device)
{
  struct termios t;
  int fd;

  fd = open (device, O_RDWR | O_NONBLOCK);
  if (fd == -1)
    return -1;
  
  if (tcgetattr (fd, &t) == -1)
    return -1;
  t.c_iflag = 0;
  t.c_oflag = 0;
  t.c_lflag = 0;
  if (tcsetattr (fd, TCSANOW, &t) == -1)
    return -1;

  return fd;
}

int
handle_device_input (int fd, int events, Tunnel *tunnel)
{
  char buf[10240];
  ssize_t n;

  if (events & POLLIN)
    {
      n = read (fd, buf, sizeof buf);
      if (n == 0 || n == -1)
	return n;

      return tunnel_write (tunnel, buf, n);
    }
  else if (events & POLLHUP)
    fprintf (stderr, "handle_device_input: PULLHUP\n");
  else if (events & POLLERR)
    fprintf (stderr, "handle_device_input: PULLERR\n");
  else if (events & POLLNVAL)
    fprintf (stderr, "handle_device_input: PULLINVAL\n");
  else
    fprintf (stderr, "handle_device_input: none of the above\n");

  return -1;
}

int
handle_tunnel_input (Tunnel *tunnel, int events, int fd)
{
  char buf[10240];
  ssize_t n;

  if (events & POLLIN)
    {
      n = tunnel_read (tunnel, buf, sizeof buf);
      if (n == 0 || n == -1)
	return n;

      return write_all (fd, buf, n);
    }
  else if (events & POLLHUP)
    fprintf (stderr, "handle_device_input: PULLHUP\n");
  else if (events & POLLERR)
    fprintf (stderr, "handle_device_input: PULLERR\n");
  else if (events & POLLNVAL)
    fprintf (stderr, "handle_device_input: PULLINVAL\n");
  else
    fprintf (stderr, "handle_device_input: none of the above\n");

  return -1;
}

void
name_and_port (const char *nameport, char **name, int *port)
{
  char *p;

  *name = strdup (nameport);
  if (*name == NULL)
    {
      fprintf (stderr, "Out of memory\n");
      exit (1);
    }

  p = strchr (*name, ':');
  if (p != NULL)
    {
      *port = atoi (p + 1);
      *p = '\0';
    }
}
