/*
common.h

Copyright (C) 1999 Lars Brinkhoff.  See COPYING for terms and conditions.
*/

#include <errno.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#include "tunnel.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

static inline ssize_t
read_all (int fd, void *buf, size_t len)
{
  ssize_t n, m;

  for (n = 0; n < len; n += m)
    {
      m = read (fd, buf + n, len - n);
      if (m == 0)
	{
	  return 0;
	}
      else if (m == -1)
	{
	  if (errno != EAGAIN)
	    return -1;
	  else
	    m = 0;
	}
    }

  return len;
}

static inline ssize_t
write_all (int fd, void *data, size_t len)
{
  ssize_t n, m;

  for (n = 0; n < len; n += m)
    {
      m = write (fd, data + n, len - n);
      if (m == 0)
	{
	  return 0;
	}
      else if (m == -1)
	{
	  if (errno != EAGAIN)
	    return -1;
	  else
	    m = 0;
	}
    }

  return len;
}

static inline int
do_connect (struct sockaddr_in *address)
{
  int fd;

  fd = socket (AF_INET, SOCK_STREAM, 0);
  if (fd == -1)
    return -1;

  if (connect (fd, address, sizeof (struct sockaddr_in)) == -1)
    {
      close (fd);
      return -1;
    }

  return fd;
}

extern int server_socket (int port);
extern int set_address (struct sockaddr_in *address,
			const char *host, int port);
extern int open_device (char *device);
extern int handle_device_input (int fd, int events, Tunnel *tunnel);
extern int handle_tunnel_input (Tunnel *tunnel, int events, int fd);
extern void name_and_port (const char *nameport, char **name, int *port);

