/*
rw.c

Copyright (C) 1999 Lars Brinkhoff.

rw is a little tool used to open a device and read/write data from
stdin/stdout from/to it.
*/

#include <stdio.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/poll.h>

#include "common.h"

int
main (int argc, char **argv)
{
  struct pollfd p[2];
  struct termios t;
  int fd;

  if (argc != 2)
    {
      fprintf (stderr, "usage: %s <device>\n", argv[0]);
      exit (1);
    }

  fd = open (argv[1], O_RDWR);
  if (fd == -1)
    {
      perror ("open");
      exit (1);
    }

  tcgetattr (fd, &t);
  t.c_iflag = 0;
  t.c_oflag = 0;
  t.c_lflag = 0;
  tcsetattr (fd, TCSANOW, &t);

  p[0].fd = 0;
  p[0].events = POLLIN;
  p[1].fd = fd;
  p[1].events = POLLIN;

  for (;;)
    {
      char buf[1024];
      ssize_t n;

      if (poll (p, 2, -1) == -1)
	{
	  perror ("poll");
	  exit (1);
	}

      if (p[0].revents & POLLIN)
	{
	  n = read (0, buf, sizeof buf);
	  if (n < 1)
	    break;
	  if (write_all (fd, buf, n) == -1)
	    perror ("write fd");
	}

      if (p[1].revents & POLLIN)
	{
	  n = read (fd, buf, sizeof buf);
	  if (n < 1)
	    break;
	  if (write_all (1, buf, n) == -1)
	    perror ("write fd");
	}
    }

  close (fd);

  return 0;
}
