/*
tt.c

Copyright (C) 1999 Lars Brinkhoff.

tt - tunnel test.  Nuff said.
*/

#include <time.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/poll.h>

#include "common.h"
#include "tunnel.h"

#define min(a,b) ((a) < (b) ? (a) : (b))

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

typedef struct
{
  char *me;
  char *file;
  char *host;
  int host_port;
  char *proxy;
  int proxy_port;
} Arguments;

static void
parse_arguments (int argc, char **argv, Arguments *arg)
{
  if (argc != 5)
    {
      fprintf (stderr, "%s: usage: %s FILE SELF PROXY PORT\n",
	       argv[0], argv[0]);
      exit (1);
    }

  srand (time (NULL));

  arg->me = argv[0];
  arg->file = argv[1];
  arg->host = argv[2];
  arg->host_port = rand () % (65535 - 1024) + 1024;
  printf ("using local port %d\n", arg->host_port);
  arg->proxy = argv[3];
  arg->proxy_port = atoi (argv[4]);
}

static int
error (int n, int err)
{
  int x;
  x = TRUE;
  if (n > 0)
    x = FALSE;
  else if (n == 0)
    printf ("connection closed\n");
  else if (errno == EAGAIN) /* n < 0 */
    x = FALSE;
  else /* n < 0 && errno != EAGAIN */
    printf ("error: %s\n", strerror (err));
  return x;
}

int
main (int argc, char **argv)
{
  int closed = FALSE;
  Tunnel *tunnel;
  int pid, child;
  Arguments arg;
  int fd;
  size_t bytes;

  parse_arguments (argc, argv, &arg);

  pid = fork ();
  child = (pid == 0);

  fd = open (arg.file, O_RDONLY);
  if (fd == -1)
    {
      fprintf (stderr, "%s: couldn't open %s: %s\n",
	       arg.me, arg.file, strerror (errno));
      exit (1);
    }

  tunnel = child ? tunnel_new_client (arg.host, arg.host_port,
				      arg.proxy, arg.proxy_port,
				      1000000) /* Content-Length */ :
                   tunnel_new_server (arg.host_port);
  if (tunnel == NULL)
    {
      fprintf (stderr, "%s: couldn't create tunnel\n", arg.me);
      exit (1);
    }

  if (child ? tunnel_connect (tunnel) == -1 :
              tunnel_accept (tunnel) == -1)
    {
      fprintf (stderr, "couldn't open tunnel: %s\n", strerror (errno));
      exit (1);
    }

  if (child)
    printf ("tunnel connected\n");
  else
    printf ("tunnel accepted\n");

  bytes = 0;
  while (!closed)
    {
      char buf[10240], buf2[sizeof buf];
      struct pollfd p;
      ssize_t n, m;

      p.fd = child ? fd : tunnel_in_fd (tunnel);
      p.events = POLLIN;
      if (poll (&p, 1, -1) == -1)
	{
	  perror ("poll");
	  exit (1);
	}

      if (child)
	{
	  n = read (fd, buf, sizeof buf);
	  closed = error (n, errno);

	  if (!closed)
	    {
	      n = tunnel_write (tunnel, buf, n);
	      closed = error (n, errno);

	      if (n > 0)
		{
		  bytes += n;
		  printf ("child: sent %d bytes, %u total\n", n, bytes);
		}
	    }
	}
      else
	{
	  n = tunnel_read (tunnel, buf, sizeof buf);
	  closed = error (n, errno);

	  if (!closed)
	    {
	      m = read_all (fd, buf2, n);
	      closed = error (n, errno);

	      if (n > 0)
		{
		  bytes += n;
		  printf ("parent: got %d bytes, %u total\n", n, bytes);
		}

	      if (n > 0 && memcmp (buf, buf2, n) != 0)
		{
		  int i;

		  printf ("error in transmission!\n");
	      
		  for (i = 0; i < n; i++)
		    {
		      if (buf[i] != buf2[i])
			{
			  printf ("at byte %d in packet, %d total\n",
				  i, bytes - n + i);
			  break;
			}
		    }
		  for (i = 0; i < min (n, 30); i++)
		    printf ("%c", buf[i]);
		  putchar ('\n');
		  for (i = 0; i < min (n, 30); i++)
		    printf ("%c", buf2[i]);
		  putchar ('\n');
		}
	    }
	}
    }

  fsync (fd);
  close (fd);
  tunnel_close (tunnel);

  return 0;
}
